import sys

from blogutils import *

from content import ContentModule
from database.dbobjects import DBPosting,DBReply,RecentReply,DBTrackback,DBCategory
from wiki.wikiutil import compile_wiki_string
import config

import ZODB

from BTrees import OOBTree
from BTrees.Length import Length

CREATE_ACTION = 1
VIEW_ACTION = 2
DELETE_ACTION = 3
EDIT_ACTION = 4
LIST_ACTION = 5
CREATE_DB_ACTION = 6
EDIT_DB_ACTION = 7
MAKE_REPLY_ACTION = 8
DELETE_REPLY_ACTION = 9
TRACKBACK_ACTION = 10
DELETE_TRACKBACK_ACTION = 11
ADMIN_ACTION = 12
AUTH_FORM_ACTION = 13
SEND_PING_FORM_ACTION = 14
SEND_PING_ACTION = 15
CREATE_DRAFT_DB = 16

action_dict = {\
    "create":CREATE_ACTION,\
    "view":VIEW_ACTION,\
	"delete":DELETE_ACTION,
    "edit":EDIT_ACTION,\
    "list":LIST_ACTION,\
    "make_reply":MAKE_REPLY_ACTION,\
    "delete_reply":DELETE_REPLY_ACTION,\
    "create_db":CREATE_DB_ACTION,\
    "edit_db":EDIT_DB_ACTION,\
    "trackback" : TRACKBACK_ACTION,\
    "delete_trackback" : DELETE_TRACKBACK_ACTION,\
    "admin" : ADMIN_ACTION,\
    "auth_form" : AUTH_FORM_ACTION,\
    "send_ping_form" : SEND_PING_FORM_ACTION,\
    "send_ping" : SEND_PING_ACTION,\
    "create_draft_db" : CREATE_DRAFT_DB,\
    }

def do_create(env, posting_id):
	return "<action name=\"create\"/>" 

def do_auth_form(env,posting_id):
	if env.session.get("is_auth") == None:
		return "<action name=\"auth_form\"/>" 
	return "<action name=\"error\"> Already Authenticated. </action>"

def do_send_ping_form(env,posting_id):
	if not env.db_facade.postings.has_key(posting_id):
		return "<action name=\"send_ping_form\"/>"
	db_posting = env.db_facade.postings[posting_id]
	replies_str = ""
	reply_seq = 0
	for db_reply in db_posting.replies:
		replies_str = replies_str + view_reply_xml_template % \
			{\
			"writer_name":db_reply.writer_name, \
			"writer_contact":db_reply.writer_contact, \
			"created_time":make_time_string(db_reply.created_time),\
			"content":db_reply.content,\
			"reply_seq":reply_seq\
			}
		reply_seq += 1

	trackbacks_str = ""
	trackback_seq = 0
	# Loop for Trackbacks
	for db_trackback in db_posting.tracked_from_list:
		trackbacks_str = trackbacks_str + view_trackback_xml_template % \
			{\
			"trackback_seq":trackback_seq,\
			"title":db_trackback.title,\
			"excerpt":db_trackback.excerpt,\
			"url":db_trackback.url,\
			"blog_name":db_trackback.blog_name,\
			}
		trackback_seq += 1

	try:
		posting_category_key = db_posting.category_key
	except AttributeError:
		posting_category_key = "__uncategoried"

	posting_str = view_posting_xml_template % \
		{\
		"content":compile_wiki_string(db_posting.content,env.request.base_uri,"wiki/",env),\
		"replies":replies_str, \
		"trackbacks":trackbacks_str,\
		"title":db_posting.title,\
		"posting_id":db_posting.posting_id,\
		"created_time":make_time_string(db_posting.created_time),\
		"rss_created_time":make_rss_time_string(db_posting.created_time),\
		"writer_name":"TEST",\
		"category_key":posting_category_key\
		}
	return "<action name=\"send_ping_form\">" +posting_str+ "</action>"

list_xml_template = """
<action name="list">
	<category-name key="%(category_key)s"><![CDATA[%(category_name)s]]></category-name>
	<posting-list>
	%(postings)s
	</posting-list>
</action>
"""

list_title_only_xml_template = """
<action name="list_title_only">
	<category-name key="%(category_key)s"><![CDATA[%(category_name)s]]></category-name>
	<posting-list>
	%(postings)s
	</posting-list>
</action>
"""

view_posting_xml_template = """
<posting id="%(posting_id)d">
	<title><![CDATA[%(title)s]]></title>
	<content><![CDATA[%(content)s]]></content>
	<replies>
		%(replies)s
	</replies>
	<trackbacks>
		%(trackbacks)s
	</trackbacks>
	<created_time>%(created_time)s</created_time>
	<rss_created_time>%(rss_created_time)s</rss_created_time>
	<category key="%(category_key)s"/>
	<writer>
		<name><![CDATA[%(writer_name)s]]></name>
	</writer>
</posting>
"""

view_reply_xml_template = """
		<reply seq="%(reply_seq)d">
			<writer>
				<name><![CDATA[%(writer_name)s]]></name>
				<contact><![CDATA[%(writer_contact)s]]></contact>
			</writer>
			<content><![CDATA[%(content)s]]></content>
			<created_time>%(created_time)s</created_time>
		</reply>
"""

view_trackback_xml_template = """
	<trackback seq="%(trackback_seq)d">
		<title><![CDATA[%(title)s]]></title>
		<excerpt><![CDATA[%(excerpt)s]]></excerpt>
		<url><![CDATA[%(url)s]]></url>
		<blog_name><![CDATA[%(blog_name)s]]></blog_name>
	</trackback>
"""

list_page_xml_template ="""
<page prev_page="%(prev_page)d" next_page="%(next_page)d" current_page="%(current_page)d"/>
"""
def do_list(env, posting_id):
	# Is there any specific category info?
	category = "__all" 
	category_name = "All"
	posting_id_list = []
	postings_str = ""
	per_page_posting = 5
	
	
	# Getting posting id list
	if env.request.form.has_key('category'):
		category = env.request.form['category'].value
		if category == "__all":
			posting_id_list = env.db_facade.postings.keys()
			for (posting_id, posting) in env.db_facade.postings.items():
				if posting.category_key != "__drafts":
					posting_id_list.append(posting_id)
		elif category == "__starred":
			category_name = "Starred"
			posting_id_list = env.db_facade.starred_postings.keys()
		elif not env.db_facade.categories.has_key(category):
			return "<action name=\"error\">There is no category for "+str(category)+"</action>"
		elif category == "__drafts":
			posting_id_list = env.db_facade.categories["__drafts"].keys()
			category_name = "Drafts"
			category_key = "__drafts"
		elif category != "__uncategoried":
			category = env.db_facade.categories[category]
			category_name = category.category_name
			category_key = category.category_key
			posting_id_list = category.category_postings.keys()
		else:
			posting_id_list = env.db_facade.categories["__uncategoried"].keys()
			category_name = "Uncategoried"
			category_key = "__uncategoried"
	elif env.request.form.has_key('day'):
		day_str = env.request.form['day'].value
		if len(day_str) != 8:
			return "<action name=\"error\"> Invalid Day String Length</action>"
		try:
			year,month,day = ParseDayFormat(day_str)
		except:
			return "<action name=\"error\"> Invalid Day Format </action>"
		if env.db_facade.daily_postings.has_key(day_str):
			posting_id_list = env.db_facade.daily_postings[day_str].keys()
		else:
			return "<action name=\"error\"> No Posting on %d/%d/%d </action>" % (year,month,day)
		if len(posting_id_list) == 0:
			return "<action name=\"error\"> No Posting on %d/%d/%d </action>" % (year,month,day)
		category_name = day_str
	elif env.request.form.has_key('month'):
		month_str = env.request.form['month'].value
		if len(month_str) != 6:
			return "<action name=\"error\"> Invalid Month String Length</action>"
		try:
			year,month = ParseMonthFormat(month_str)
		except:
			return "<action name=\"error\"> Invalid month Format </action>"
		if env.db_facade.monthly_postings.has_key(month_str):
			posting_id_list = env.db_facade.monthly_postings[month_str].keys()
		else:
			return "<action name=\"error\"> No Posting on %d/%d</action>" % (year,month)
		if len(posting_id_list) == 0:
			return "<action name=\"error\"> No Posting on %d/%d</action>" % (year,month)
		category_name = month_str
	else:
		for (posting_id, posting) in env.db_facade.postings.items():
			if posting.category_key != "__drafts":
				posting_id_list.append(posting_id)
		
	if env.request.form.has_key('page'):
		page = env.request.form['page'].value
		page = int(page)
	else:
		page = 0
	
	
	is_title_only = False
	if env.request.form.has_key('title_only'):
		title_only = env.request.form['title_only'].value
		if title_only == "yes":
			is_title_only = True
	
	if is_title_only == True:
		per_page_posting = len(posting_id_list)

	# Make Reverse Iterator
	reverse_iter = reverse_traverse(posting_id_list)	
	num_of_total = len(posting_id_list)
	num_of_displayed = 0
	num_of_skipped = 0

	# Page Skip
	for n in range(per_page_posting*page):
		try:
			reverse_iter.next()
			num_of_skipped += 1
		except StopIteration:
			return "<action name=\"error\">There is no posting for this page: "+category_name+"</action>"
	

	# Loop for Postings
	for n in range(per_page_posting):
		try:
			posting_id = reverse_iter.next()
		except StopIteration:
			break
		if not env.db_facade.postings.has_key(posting_id):
			num_of_total -= 1
			continue
		num_of_displayed+=1
		db_posting = env.db_facade.postings[posting_id]
		replies_str = ""
		reply_seq = 0
		# Loop for Replies
		for db_reply in db_posting.replies:
			replies_str = replies_str + view_reply_xml_template % \
				{\
				"writer_name":db_reply.writer_name, \
				"writer_contact":db_reply.writer_contact, \
				"created_time":make_time_string(db_reply.created_time),\
				"content":db_reply.content,\
				"reply_seq":reply_seq\
				}
			reply_seq += 1
		
		trackbacks_str = ""
		trackback_seq = 0
		# Loop for Trackbacks
		for db_trackback in db_posting.tracked_from_list:
			trackbacks_str = trackbacks_str + view_trackback_xml_template % \
				{\
				"trackback_seq":trackback_seq,\
				"title":db_trackback.title,\
				"excerpt":db_trackback.excerpt,\
				"url":db_trackback.url,\
				"blog_name":db_trackback.blog_name,\
				}
			trackback_seq += 1

		try:
			posting_category_key = db_posting.category_key
		except AttributeError:
			posting_category_key = "__uncategoried"
		
		# export make posting
		postings_str = postings_str + view_posting_xml_template % \
			{"content":compile_wiki_string(db_posting.content,env.request.base_uri,"wiki/",env),\
			"replies":replies_str, \
			"trackbacks":trackbacks_str,\
			"title":db_posting.title,\
			"posting_id":db_posting.posting_id,\
			"created_time":make_time_string(db_posting.created_time),\
			"rss_created_time":make_rss_time_string(db_posting.created_time),\
			"writer_name":config.blog_writer,\
			"category_key":posting_category_key\
			}
	if num_of_displayed == 0:
		return "<action name=\"error\">There is no posting for category: "+category_name+"</action>"

	# page processing
	current_page = page
	left = num_of_total - num_of_skipped - num_of_displayed
	if left > 0:
		next_page = page + 1
	else:
		next_page = -1
	if num_of_skipped > 0:
		prev_page = page - 1
	else:
		prev_page = -1

	page_str = list_page_xml_template % {"prev_page":prev_page,"next_page":next_page,"current_page":current_page}

	if category == "__uncategoried" or category == "__all" or category == "__starred" or category == "__drafts":
		category_key_data = category
	else:
		category_key_data = category.category_key
	if is_title_only == False:
		return page_str + list_xml_template % \
			{\
			"category_name" : category_name, \
			"category_key" : category_key_data,\
			"postings" : postings_str,\
			}
	else:
		return page_str + list_title_only_xml_template % \
			{\
			"category_name" : category_name, \
			"category_key" : category_key_data,\
			"postings" : postings_str,\
			}


def do_view(env, posting_id):
	if not env.db_facade.postings.has_key(posting_id):
		return "<action name=\"error\">There is no posting for "+str(posting_id)+"</action>"
	db_posting = env.db_facade.postings[posting_id]
	replies_str = ""
	reply_seq = 0
	for db_reply in db_posting.replies:
		replies_str = replies_str + view_reply_xml_template % \
			{\
			"writer_name":db_reply.writer_name, \
			"writer_contact":db_reply.writer_contact, \
			"created_time":make_time_string(db_reply.created_time),\
			"content":db_reply.content,\
			"reply_seq":reply_seq\
			}
		reply_seq += 1

	trackbacks_str = ""
	trackback_seq = 0
	# Loop for Trackbacks
	for db_trackback in db_posting.tracked_from_list:
		trackbacks_str = trackbacks_str + view_trackback_xml_template % \
			{\
			"trackback_seq":trackback_seq,\
			"title":db_trackback.title,\
			"excerpt":db_trackback.excerpt,\
			"url":db_trackback.url,\
			"blog_name":db_trackback.blog_name,\
			}
		trackback_seq += 1

	try:
		posting_category_key = db_posting.category_key
	except AttributeError:
		posting_category_key = "__uncategoried"

	posting_str = view_posting_xml_template % \
		{\
		"content":compile_wiki_string(db_posting.content,env.request.base_uri,"wiki/",env),\
		"replies":replies_str, \
		"trackbacks":trackbacks_str,\
		"title":db_posting.title,\
		"posting_id":db_posting.posting_id,\
		"created_time":make_time_string(db_posting.created_time),\
		"rss_created_time":make_rss_time_string(db_posting.created_time),\
		"writer_name":"TEST",\
		"category_key":posting_category_key\
		}
	return "<action name=\"view\">" +posting_str+ "</action>"
	    
def do_edit(env, posting_id):
	if not env.db_facade.postings.has_key(posting_id):
		return "<action name=\"error\">There is no posting for "+str(posting_id)+"</action>"
	db_posting = env.db_facade.postings[posting_id]
	replies_str = ""
	trackbacks_str = ""
	reply_seq = 0
	for db_reply in db_posting.replies:
		replies_str = view_reply_xml_template % \
			{\
			"writer_name":db_reply.writer_name, \
			"writer_contact":db_reply.writer_contact, \
			"created_time":make_time_string(db_reply.created_time),\
			"content":db_reply.content,\
			"reply_seq":reply_seq\
			}
		reply_seq += 1
		trackbacks_str = ""
		trackback_seq = 0
		# Loop for Trackbacks
		for db_trackback in db_posting.tracked_from_list:
			trackbacks_str = trackbacks_str + view_trackback_xml_template % \
				{\
				"trackback_seq":trackback_seq,\
				"title":db_trackback.title,\
				"excerpt":db_trackback.excerpt,\
				"url":db_trackback.url,\
				"blog_name":db_trackback.blog_name,\
				}
			trackback_seq += 1
	try:
		posting_category_key = db_posting.category_key
	except AttributeError:
		posting_category_key = "__uncategoried"
	posting_str = view_posting_xml_template % \
		{\
		"content":db_posting.content,\
		"replies":replies_str, \
		"title":db_posting.title,\
		"posting_id":db_posting.posting_id,\
		"trackbacks":trackbacks_str,\
		"created_time":make_time_string(db_posting.created_time),\
		"writer_name":"TEST",\
		"category_key":posting_category_key,\
		"rss_created_time":make_rss_time_string(db_posting.created_time),\
		}
	return "<action name=\"edit\">" +posting_str+ "</action>"

def do_edit_db(env,posting_id):
	if not env.db_facade.postings.has_key(posting_id):
		return "<action name=\"error\">There is no posting for "+str(posting_id)+"</action>"
	if not env.request.form.has_key('title'):
		return "<action name=\"error\">There is no title field</action>"
	if not env.request.form.has_key('content'):
		return "<action name=\"error\">There is no content field</action>"
	if not env.request.form.has_key('category'):
		return "<action name=\"error\">There is no category field</action>"
	if not env.request.form.has_key('old_category_key'):
		return "<action name=\"error\">There is no old category field</action>"

	title = env.request.form['title'].value
	content = env.request.form['content'].value
	category_key = env.request.form['category'].value
	old_category_key = env.request.form['old_category_key'].value

	db_posting = env.db_facade.postings[posting_id]
	db_posting.title = title
	db_posting.content = content
	
	if category_key != old_category_key:
		if category_key == '__uncategoried':
			category_postings_to_add = env.db_facade.categories['__uncategoried']
		elif category_key == '__drafts':
			category_postings_to_add = env.db_facade.categories['__drafts']
		else:
			category_postings_to_add = env.db_facade.categories[category_key].category_postings
		
		if old_category_key == '__uncategoried':
			category_postings_to_del = env.db_facade.categories['__uncategoried']
		elif old_category_key == '__drafts':
			category_postings_to_del = env.db_facade.categories['__drafts']
		else:
			category_postings_to_del = env.db_facade.categories[old_category_key].category_postings

		# move from drafts to non-drafts category
		if category_key != '__drafts' and old_category_key == '__drafts':

			# delete in uncategoried category
			if db_posting.category_key == '__uncategoried':
				if env.db_facade.categories["__uncategoried"].has_key(db_posting.posting_id):
					del env.db_facade.categories["__uncategoried"][db_posting.posting_id]
			elif db_posting.category_key == '__drafts':
				if env.db_facade.categories["__drafts"].has_key(db_posting.posting_id):
					del env.db_facade.categories["__drafts"][db_posting.posting_id]
			# or delete in category
			elif env.db_facade.categories.has_key(db_posting.category_key):
				category = env.db_facade.categories[db_posting.category_key]
				if category.category_postings.has_key(db_posting.posting_id):
					del category.category_postings[db_posting.posting_id]
				
			# delete in daily_posting
			day_key = make_daily_key(db_posting.created_time)
			if env.db_facade.daily_postings.has_key(day_key):
				if env.db_facade.daily_postings[day_key].has_key(db_posting.posting_id):
					del env.db_facade.daily_postings[day_key][db_posting.posting_id]
		
			# delete in monthly_posting
			month_key = make_monthly_key(db_posting.created_time)
			if env.db_facade.monthly_postings.has_key(month_key):
				if env.db_facade.monthly_postings[month_key].has_key(db_posting.posting_id):
					del env.db_facade.monthly_postings[month_key][db_posting.posting_id]
			
			# delete in starred_postings
			if db_posting.posting_id in env.db_facade.starred_postings.keys():
				del env.db_facade.starred_postings[db_posting.posting_id]
			
			# delete old thing
			del  env.db_facade.postings[db_posting.posting_id]
			
			idx = 0
			for recent_reply in env.db_facade.recent_blog_reply_infos:
				if recent_reply.posting_id == posting_id:
					del env.db_facade.recent_blog_reply_infos[idx]
				idx = idx + 1
			del db_posting
			
			env.db_facade.Commit()
			get_transaction().commit()
			
			# make new thing
			db_posting = DBPosting(None,title,content,category_key,env.db_facade.db)

			# process daily posting
			day_key = make_daily_key(db_posting.created_time)
			if not env.db_facade.daily_postings.has_key(day_key):
				env.db_facade.daily_postings[day_key] = OOBTree.OOBTree()
			env.db_facade.daily_postings[day_key][db_posting.posting_id] = db_posting
		
			# process monthly posting
			month_key = make_monthly_key(db_posting.created_time)
			if not env.db_facade.monthly_postings.has_key(month_key):
				env.db_facade.monthly_postings[month_key] = OOBTree.OOBTree()
			env.db_facade.monthly_postings[month_key][db_posting.posting_id] = db_posting
			
			#process starred
			if Check(env.request.form,"is_starred"):
				if env.request.form["is_starred"].value == 'on':
					env.db_facade.starred_postings[db_posting.posting_id] = db_posting

			# store new thing
			env.db_facade.postings[db_posting.posting_id] = db_posting
			category_postings_to_add[db_posting.posting_id] = db_posting
		
		# move from non-drafts to drafts category
		elif category_key == '__drafts' and old_category_key != '__drafts':
			# delete old thing
			# delete in uncategoried category
			if db_posting.category_key == '__uncategoried':
				if env.db_facade.categories["__uncategoried"].has_key(db_posting.posting_id):
					del env.db_facade.categories["__uncategoried"][db_posting.posting_id]
			elif db_posting.category_key == '__drafts':
				print "checking %s in category Drafts" % (str(db_posting.posting_id))
				if env.db_facade.categories["__drafts"].has_key(db_posting.posting_id):
					print "delete %s in category Drafts" % (str(db_posting.posting_id))
					del env.db_facade.categories["__drafts"][db_posting.posting_id]
			# or delete in category
			elif env.db_facade.categories.has_key(db_posting.category_key):
				category = env.db_facade.categories[db_posting.category_key]
				if category.category_postings.has_key(db_posting.posting_id):
					del category.category_postings[db_posting.posting_id]
				
			# delete in daily_posting
			day_key = make_daily_key(db_posting.created_time)
			if env.db_facade.daily_postings.has_key(day_key):
				if env.db_facade.daily_postings[day_key].has_key(db_posting.posting_id):
					del env.db_facade.daily_postings[day_key][db_posting.posting_id]
		
			# delete in monthly_posting
			month_key = make_monthly_key(db_posting.created_time)
			if env.db_facade.monthly_postings.has_key(month_key):
				if env.db_facade.monthly_postings[month_key].has_key(db_posting.posting_id):
					del env.db_facade.monthly_postings[month_key][db_posting.posting_id]
			
			# delete in starred_postings
			if db_posting.posting_id in env.db_facade.starred_postings.keys():
				del env.db_facade.starred_postings[db_posting.posting_id]
			
			del  env.db_facade.postings[db_posting.posting_id]
			idx = 0
			for recent_reply in env.db_facade.recent_blog_reply_infos:
				if recent_reply.posting_id == posting_id:
					del env.db_facade.recent_blog_reply_infos[idx]
				idx = idx + 1
			del db_posting
			env.db_facade.Commit()
			get_transaction().commit()

			
			# make new thing
			db_posting = DBPosting(None,title,content,category_key,env.db_facade.db)

			# store new thing
			env.db_facade.postings[db_posting.posting_id] = db_posting
			category_postings_to_add[db_posting.posting_id] = db_posting
		else:
			db_posting.category_key = category_key
			if category_postings_to_del.has_key(db_posting.posting_id):
				del category_postings_to_del[db_posting.posting_id]
			category_postings_to_add[db_posting.posting_id] = db_posting
	
	
	# delete in starred_postings
	if db_posting.posting_id in env.db_facade.starred_postings.keys():
		del env.db_facade.starred_postings[db_posting.posting_id]

	# process starred
	if Check(env.request.form,"is_starred"):
		if env.request.form["is_starred"].value == 'on':
			env.db_facade.starred_postings[db_posting.posting_id] = db_posting
	
	env.db_facade.Commit()
	get_transaction().commit()
	return "<action name=\"edit_db\" posting-id=\"%d\"/>" % (db_posting.posting_id)

def Check(form, key):
	if form.has_key(key) and form[key].value != "":
		return True
	return False

def send_ping(uri, blog_name, posting_uri, title, excerpt):
	try:
		server_uri = uri
		host_idx = uri.find("http://")
		if host_idx < 0:
			return "no http://"
		http_removed = uri[host_idx+7:]
		host_port_end_idx = http_removed.find("/")
		server_url = http_removed[host_port_end_idx:]
		host_port = http_removed[:host_port_end_idx]
		
		import socket
		socket.setdefaulttimeout(3)
		
		import httplib,urllib
		params = {"excerpt" : excerpt,"blog_name" : blog_name,"url" : posting_uri,"title" : title}
		headers = {"Content-Type":"application/x-www-form-urlencoded"}
		url_encoded_params = urllib.urlencode(params)
		
		conn = httplib.HTTPConnection(host_port)
		conn.request("POST",server_url,url_encoded_params,headers)
		response = conn.getresponse()
		response.read()
		conn.close()
	except:
		return "Error"
	return True

def do_create_db(env,posting_id):
	if not env.request.form.has_key('title'):
		return "<action name=\"error\">There is no title field</action>"
	if not env.request.form.has_key('content'):
		return "<action name=\"error\">There is no content field</action>"
	if not env.request.form.has_key('category'):
		return "<action name=\"error\">There is no category field</action>"
	title = env.request.form['title'].value
	content = env.request.form['content'].value
	category_key = env.request.form['category'].value

	if category_key == '__uncategoried':
		category_postings_to_add = env.db_facade.categories['__uncategoried']
	elif category_key == '__drafts':
		category_postings_to_add = env.db_facade.categories['__drafts']
	else:
		category_postings_to_add = env.db_facade.categories[category_key].category_postings
	
	posting = DBPosting(None,title,content,category_key,env.db_facade.db)
	env.db_facade.postings[posting.posting_id] = posting
	category_postings_to_add[posting.posting_id] = posting
	
	if category_key != '__drafts':
		# process daily posting
		day_key = make_daily_key(posting.created_time)
		if not env.db_facade.daily_postings.has_key(day_key):
			env.db_facade.daily_postings[day_key] = OOBTree.OOBTree()
		env.db_facade.daily_postings[day_key][posting.posting_id] = posting
	
		# process monthly posting
		month_key = make_monthly_key(posting.created_time)
		if not env.db_facade.monthly_postings.has_key(month_key):
			env.db_facade.monthly_postings[month_key] = OOBTree.OOBTree()
		env.db_facade.monthly_postings[month_key][posting.posting_id] = posting
		
		#process starred
		if Check(env.request.form,"is_starred"):
			if env.request.form["is_starred"].value == 'on':
				env.db_facade.starred_postings[posting.posting_id] = posting
	
	# send trackback ping if assigned
	if Check(env.request.form,"trackback_uri"):
		trackback_server = env.request.form["trackback_uri"].value
		posting_uri = env.request.base_uri + "/" + env.request.application_id + "/" + str(posting.posting_id)
		excerpt = content
		if len(excerpt) > 255:
			excerpt = excerpt[:252] + "..."
		result = send_ping(trackback_server, config.site_name, posting_uri, title,excerpt)
		if result != True:
			pass
	env.db_facade.Commit()
	get_transaction().commit()
	return "<action name=\"create_db\" posting-id=\"%d\"/>" % (posting.posting_id)

		
def do_make_reply(env,posting_id):
	if not Check(env.request.form,"writer_name"):
		return "<action name=\"error\">There is no writer name field</action>"
	if not Check(env.request.form,"passwd"):
		return "<action name=\"error\">There is no password field</action>"
	if not Check(env.request.form,"content"):
		return "<action name=\"error\">There is no content field</action>"
	if not Check(env.request.form,"writer_contact"):
		return "<action name=\"error\">There is writer contact field</action>"
	if not Check(env.request.form,"posting_id"):
		return "<action name=\"error\">There is posting id field</action>"

	import md5
	password = md5.new(env.request.form["passwd"].value).hexdigest()

	new_reply = DBReply(\
		env.request.form["writer_name"].value, \
		env.request.form["writer_contact"].value, \
		env.request.form["content"].value, \
		password, \
		env.db_facade.db,\
		)
	db_posting = env.db_facade.postings[int(env.request.form["posting_id"].value)]
	reply_seq = db_posting.AddReply(new_reply,env.db_facade.db)
	if len(env.db_facade.recent_blog_reply_infos) > 10:
		del env.db_facade.recent_blog_reply_infos[:len(env.db_facade.recent_blog_reply_infos) - 10]
	env.db_facade.recent_blog_reply_infos.append(RecentReply(db_posting.posting_id,reply_seq,"["+env.request.form["writer_name"].value+"] "+env.request.form["content"].value))
	env.db_facade.Commit()
	get_transaction().commit()
	return "<action name=\"make_reply\" posting-id=\"%d\"/>" % (db_posting.posting_id)

def do_delete(env,posting_id):
	posting = env.db_facade.postings[posting_id]
	
	# delete in uncategoried category
	category_key = posting.category_key
	if category_key == '__uncategoried':
		if env.db_facade.categories["__uncategoried"].has_key(posting_id):
			del env.db_facade.categories["__uncategoried"][posting_id]
	elif category_key == '__drafts':
		if env.db_facade.categories["__drafts"].has_key(posting_id):
			del env.db_facade.categories["__drafts"][posting_id]
	# or delete in category
	elif env.db_facade.categories.has_key(category_key):
		category = env.db_facade.categories[category_key]
		if category.category_postings.has_key(posting_id):
			del category.category_postings[posting_id]
		
	# delete in daily_posting
	day_key = make_daily_key(posting.created_time)
	if env.db_facade.daily_postings.has_key(day_key):
		if env.db_facade.daily_postings[day_key].has_key(posting.posting_id):
			del env.db_facade.daily_postings[day_key][posting.posting_id]

	# delete in monthly_posting
	month_key = make_monthly_key(posting.created_time)
	if env.db_facade.monthly_postings.has_key(month_key):
		if env.db_facade.monthly_postings[month_key].has_key(posting.posting_id):
			del env.db_facade.monthly_postings[month_key][posting.posting_id]
	
	# delete in starred_postings
	if posting.posting_id in env.db_facade.starred_postings.keys():
		del env.db_facade.starred_postings[posting.posting_id]
	
	del  env.db_facade.postings[posting_id]
	idx = 0
	for recent_reply in env.db_facade.recent_blog_reply_infos:
		if recent_reply.posting_id == posting_id:
			del env.db_facade.recent_blog_reply_infos[idx]
		idx = idx + 1
	return "<action name=\"delete\"/>"

def do_delete_reply_form(env,posting_id):
	return "<action name=\"delete_reply_form\"/>"
	
def do_delete_reply(env,posting_id):
	db_posting = env.db_facade.postings[posting_id]
	if not Check(env.request.form,"reply_seq"):
		return "<action name=\"error\">There is no reply sequence field</action>"

	if env.session.get("is_auth") == None:
		if not Check(env.request.form,"password"):
			return "<action name=\"error\">Empty Password</action>"

		import md5
		input_password = md5.new(env.request.form["password"].value).hexdigest()

		reply_seq = int(env.request.form["reply_seq"].value)
		stored_password = db_posting.replies[reply_seq].passwd
		if input_password == stored_password:
			del db_posting.replies[reply_seq]
			idx = 0
			for recent_reply in env.db_facade.recent_blog_reply_infos:
				if recent_reply.posting_id == posting_id and recent_reply.reply_seq == reply_seq:
					del env.db_facade.recent_blog_reply_infos[idx]
				idx = idx + 1
		return "<action name=\"delete_reply\" posting-id=\"%d\"/>" % (db_posting.posting_id)

	else:
		reply_seq = int(env.request.form["reply_seq"].value)
		del db_posting.replies[reply_seq]
		idx = 0
		for recent_reply in env.db_facade.recent_blog_reply_infos:
			if recent_reply.posting_id == posting_id and recent_reply.reply_seq == reply_seq:
				del env.db_facade.recent_blog_reply_infos[idx]
			idx = idx + 1
		return "<action name=\"delete_reply\" posting-id=\"%d\"/>" % (db_posting.posting_id)

def do_delete_trackback(env,posting_id):
	db_posting = env.db_facade.postings[posting_id]
	if not Check(env.request.form,"trackback_seq"):
		return "<action name=\"error\">There is no trackback sequence field</action>"
	trackback_seq = int(env.request.form["trackback_seq"].value)
	del db_posting.tracked_from_list[trackback_seq]
	idx = 0
	#for recent_reply in env.db_facade.recent_blog_reply_infos:
	#	if recent_reply.posting_id == posting_id and recent_reply.reply_seq == reply_seq:
	#		del env.db_facade.recent_blog_reply_infos[idx]
	#	idx = idx + 1
	return "<action name=\"delete_trackback\" posting-id=\"%d\"/>" % (db_posting.posting_id)

trackback_success_template = """<?xml version="1.0" encoding="utf-8"?>
<response>
	<error>0</error>
</response>
"""

trackback_fail_template = """<?xml version="1.0" encoding="utf-8"?>
<response>
	<error>1</error> 
	<message>%s</message>
</response>
"""

def do_trackback(env,posting_id):
	env.response.render_mode = "truebypass"
	env.response.header["Content-Type"] = "text/xml"
	if not env.db_facade.postings.has_key(posting_id):
		return trackback_fail_template % ("No Posting %s" % (posting_id))
	db_posting = env.db_facade.postings[posting_id]
	if not Check(env.request.form,"title"):
		return trackback_fail_template % ("No title field")
	if not Check(env.request.form,"excerpt"):
		return trackback_fail_template % ("No excerpt field")
	if not Check(env.request.form,"url"):
		return trackback_fail_template % ("No url field")
	if not Check(env.request.form,"blog_name"):
		return trackback_fail_template % ("No blog_name field")
	title = env.request.form["title"].value
	excerpt = env.request.form["excerpt"].value
	url_info = env.request.form["url"].value
	blog_name = env.request.form["blog_name"].value
	new_trackback = DBTrackback(title,excerpt,url_info,blog_name)
	db_posting.tracked_from_list.append(new_trackback)
	env.db_facade.Commit()
	get_transaction().commit()
	return trackback_success_template

def do_admin_create_category(env):
	return "<action name=\"admin\" admin_action=\"create_category\"/>" 
def do_admin_create_category_db(env):
	if not Check(env.request.form,"category_key"):
		return "<action name=\"error\">There are no category key field </action>" 
	elif not Check(env.request.form,"category_name"):
		return "<action name=\"error\">There are no category name field </action>" 
	elif not Check(env.request.form,"category_desc"):
		return "<action name=\"error\">There are no category desc field </action>" 
		
	category_key = env.request.form["category_key"].value
	category_name = env.request.form["category_name"].value
	category_desc = env.request.form["category_desc"].value
	
	if not env.db_facade.categories.has_key(category_key):
		env.db_facade.categories[category_key] = DBCategory(category_key,category_name,category_desc,env.db_facade.db)
		env.db_facade.Commit()
		return "<action name=\"admin\" admin_action=\"create_category_db\"/>"
	else:
		return "<action name=\"error\">There exists category %s </action>" % (category_key)
	return "<action name=\"admin\" admin_action=\"create_category_db\"/>" 

def do_admin_edit_category(env):
	if env.request.form.has_key("category"):
		category = env.request.form["category"].value
		if env.db_facade.categories.has_key(category):
			return "<action name=\"admin\" admin_action=\"edit_category\"><category-to-edit key=\"%s\"/></action>" % (category)
		else:
			return "<action name=\"error\">There are no category like %s </action>" % (category)
	else:
		return "<action name=\"error\">There are no category field </action>" 

def do_admin_edit_category_db(env):
	if not Check(env.request.form,"category_key"):
		return "<action name=\"error\">There are no category key field </action>" 
	elif not Check(env.request.form,"category_name"):
		return "<action name=\"error\">There are no category name field </action>" 
	elif not Check(env.request.form,"category_desc"):
		return "<action name=\"error\">There are no category desc field </action>" 
		
	category_key = env.request.form["category_key"].value
	category_name = env.request.form["category_name"].value
	category_desc = env.request.form["category_desc"].value
	
	if env.db_facade.categories.has_key(category_key):
		category = env.db_facade.categories[category_key]
		category.category_name = category_name
		category.category_description = category_desc
		env.db_facade.Commit()
		return "<action name=\"admin\" admin_action=\"edit_category_db\"/>"
	else:
		return "<action name=\"error\">There are no category like %s </action>" % (category)

def do_admin_delete_category(env):
	if env.request.form.has_key("category"):
		category = env.request.form["category"].value
		if env.db_facade.categories.has_key(category):
			return "<action name=\"admin\" admin_action=\"delete_category\"><category-to-delete key=\"%s\"/></action>" % (category)
		else:
			return "<action name=\"error\">There are no category like %s </action>" % (category)
	else:
		return "<action name=\"error\">There are no category field </action>" 

def do_admin_delete_category_db(env):
	if not Check(env.request.form,"category_key"):
		return "<action name=\"error\">There are no category key field </action>" 
		
	category_key = env.request.form["category_key"].value
	
	if env.db_facade.categories.has_key(category_key):
		category = env.db_facade.categories[category_key]
		uncategoried_category = env.db_facade.categories["__uncategoried"]
		for (key,posting) in category.category_postings.items():
			uncategoried_category[posting.posting_id] = posting
			del category.category_postings[posting.posting_id]
		del env.db_facade.categories[category_key]
		env.db_facade.Commit()
		return "<action name=\"admin\" admin_action=\"delete_category_db\"/>"
	else:
		return "<action name=\"error\">There are no category like %s </action>" % (category)
	return "<action name=\"admin\" admin_action=\"delete_category_db\"/>"

def do_admin(env,posting_id):
	admin_action = "default"
	if env.request.form.has_key("admin_action"):
		admin_action = env.request.form["admin_action"].value
	if admin_action == "default":
		return "<action name=\"admin\" admin_action=\"default\"/>"
	elif admin_action == "create_category":
		return do_admin_create_category(env)
	elif admin_action == "create_category_db":
		return do_admin_create_category_db(env)
	elif admin_action == "edit_category":
		return do_admin_edit_category(env)
	elif admin_action == "edit_category_db":
		return do_admin_edit_category_db(env)
	elif admin_action == "delete_category":
		return do_admin_delete_category(env)
	elif admin_action == "delete_category_db":
		return do_admin_delete_category_db(env)
	else:
		return "<action name=\"error\">There is no admin command</action>"

def do_send_ping(env,posting_id):
	if not Check(env.request.form,"target_uri"):
		return "<action name=\"error\">There is no target_uri field</action>"
	elif not Check(env.request.form,"local_uri"):
		return "<action name=\"error\">There is no local_uri field</action>"
	elif not Check(env.request.form,"site_name"):
		return "<action name=\"error\">There is no site_name field</action>"
	elif not Check(env.request.form,"title"):
		return "<action name=\"error\">There is no title field</action>"
	elif not Check(env.request.form,"content"):
		return "<action name=\"error\">There is no content field</action>"
	
	target_uri = env.request.form["target_uri"].value
	local_uri = env.request.form["local_uri"].value
	site_name = env.request.form["site_name"].value
	title = env.request.form["title"].value
	content = env.request.form["content"].value
	if send_ping(target_uri, site_name, local_uri, title, content):
		return "<action name=\"error\">send ping success</action>"
	else:
		return "<action name=\"error\">send ping fail</action>"
	
embedded_dispatcher_dict = {\
	CREATE_ACTION: do_create,\
	VIEW_ACTION: do_view,\
	DELETE_ACTION: do_delete,\
	EDIT_ACTION: do_edit,\
	LIST_ACTION: do_list,\
	CREATE_DB_ACTION: do_create_db,\
	EDIT_DB_ACTION: do_edit_db,\
	MAKE_REPLY_ACTION: do_make_reply,\
	DELETE_REPLY_ACTION: do_delete_reply,\
	TRACKBACK_ACTION: do_trackback,\
	DELETE_TRACKBACK_ACTION: do_delete_trackback,\
	ADMIN_ACTION: do_admin,\
	AUTH_FORM_ACTION: do_auth_form,\
	SEND_PING_FORM_ACTION: do_send_ping_form,\
	SEND_PING_ACTION: do_send_ping,\
	}

protected_action_list = [EDIT_ACTION,CREATE_ACTION,CREATE_DB_ACTION,EDIT_DB_ACTION,DELETE_ACTION,DELETE_TRACKBACK_ACTION, ADMIN_ACTION]

category_xml_template = """
<category key="%(key)s" posting-size="%(posting_size)d">
	<name><![CDATA[%(name)s]]></name>
	<desc><![CDATA[%(desc)s]]></desc>
</category>
"""

calendar_xml_template = """
<calendar year="%(year)d" month="%(month)d" linkyear="%(linkyear)s" linkmonth="%(linkmonth)s">
	%(weeks)s
</calendar>
"""

calendar_week_xml_template = """
	<week seq="%(week_seq)d">
		%(days)s
	</week>"""
	
calendar_day_xml_template = """
		<day noday="no" num="%(day)d" weekday="%(weekday)s" weekday_num="%(weekday_num)d" postings="%(postings)s" linkday="%(linkday)s"/>"""

calendar_noday_xml_template = """
		<day noday="yes"/>"""
monthly_archives_xml_template = """
	<monthly-archives>
		%(archives)s
	</monthly-archives>
"""

monthly_archive_xml_template = """
		<monthly-archive id="%(id)s" len="%(length)d"/>"""

class BlogModule(ContentModule):
	def GetTagName(self):
		return "blog"
	def Run(self, application_id, env, module_uri):
		env.request.application_id = application_id
		posting_num = None
		list_mode = False
		list_from = None
		action = LIST_ACTION
		
		category = ""
		if module_uri == "" or module_uri == "/":
			# It's Default Mode. list up first page
			list_mode = True
		elif module_uri.startswith("/"):
			try:
				posting_num = int(module_uri[1:])
				action = VIEW_ACTION
			except ValueError:
				list_mode = True
				list_from = None
		else:
			pagename = module_uri

		action_function = embedded_dispatcher_dict[action]
		dbposting=None
		
		if env.request.form.has_key("action"):
			# We received some action!
			# check action.
			if action_dict.has_key(env.request.form["action"].value):
				# it's embedded action
				action = action_dict[env.request.form["action"].value]
				action_function = embedded_dispatcher_dict[action]
			else:
				# some external actions?
				pass
		if env.request.header.has_key("QUERY_STRING"):
			query_string = env.request.header["QUERY_STRING"]
			if query_string == "action=trackback":
				action_function = do_trackback
		# Now Add Call Info
		call_info = "<module-call-info><application-id>%s</application-id><module_uri>%s</module_uri></module-call-info>"\
			% (application_id, module_uri)
		categories = "<categories total-size=\"%(total_size)d\">" % {"total_size":len(env.db_facade.postings)}
		for (key,category) in env.db_facade.categories.items():
			if key == "__uncategoried":
				categories += category_xml_template % {"name":"Uncategoried", "desc" : "Not Categorized Postings", "key":key,"posting_size":len(category)}
			elif key == "__drafts":
				pass
			else:
				categories += category_xml_template % {"name":category.category_name, "desc" : category.category_description, "key":key,"posting_size":len(category.category_postings)}
		categories += "</categories>"
		recent_reply_xml_template = 	"""
		<recent-reply posting-id="%d" reply-seq="%d"><![CDATA[%s]]></recent-reply>
		"""
		recent_replies = "<recent-replies>"
		for recent_reply in env.db_facade.recent_blog_reply_infos:
			recent_replies += recent_reply_xml_template % (recent_reply.posting_id,recent_reply.reply_seq,recent_reply.description)
		recent_replies += "</recent-replies>"
	
		if env.session.get("is_auth") == None and (action in protected_action_list):
			return call_info + categories+"<action name=\"error\">You have no privilege!!</action>"
		
		if env.request.form.has_key("render"):
			if env.request.form["render"].value == "rss" or env.request.form["render"].value == "bypass":
				env.response.render_mode = env.request.form["render"].value

		current_year = -1
		current_month = -1
	
		if env.request.form.has_key('day'):
			day_str = env.request.form['day'].value
			if len(day_str) != 8:
				current_year = -1
				current_month = -1
			try:
				current_year,current_month,current_day = ParseDayFormat(day_str)
			except:
				current_year = -1
				current_month = -1
		elif env.request.form.has_key('month'):
			month_str = env.request.form['month'].value
			if len(month_str) != 6:
				current_year = -1
				current_month = -1
			try:
				current_year,current_month = ParseMonthFormat(month_str)
			except:
				current_year = -1
				current_month = -1

		if current_year == -1 or current_month == -1:
			import time
			current_year = time.localtime(time.time())[0]
			current_month = time.localtime(time.time())[1]
	
		
		month_data = make_calender_data(current_year ,current_month)
		weeks = ""
		week_seq = 0
		for week in month_data:
			days = ""
			wday_idx = 0
			for day in week:
				if day == None:
					days += calendar_noday_xml_template
				else:
					daily_key = "%04d%02d%02d" % (current_year,current_month,day)
					is_postings = "no"
					if env.db_facade.daily_postings.has_key(daily_key):
						if len(env.db_facade.daily_postings[daily_key]) > 0:
							is_postings = "yes"
					days += calendar_day_xml_template % {\
						"day" : day,\
						"linkday" : "%02d"%day,\
						"weekday" : weekday_strings[wday_idx],\
						"weekday_num" : wday_idx,\
						"postings" : is_postings,\
						}
				wday_idx += 1
			weeks += calendar_week_xml_template % {\
				"week_seq": week_seq,\
				"days": days,\
				}
			week_seq += 1
		
		calendar_xml = calendar_xml_template % {\
			"year":current_year,\
			"month":current_month,\
			"linkyear":"%02d"%current_year,\
			"linkmonth":"%02d"%current_month,\
			"weeks": weeks,\
			}
		
		monthly_archives=""
		
		for key,archive in env.db_facade.monthly_postings.items():
			if len(archive) > 0:
				monthly_archives += monthly_archive_xml_template % {\
					"id" : key,\
					"length" : len(archive),\
				}	
		monthly_archives_xml = monthly_archives_xml_template % {\
			"archives" : monthly_archives,\
		}
		
		# Now runs action
		action_result = action_function(env,posting_num)
		if env.response.render_mode == "truebypass":
			return action_result
		return  monthly_archives_xml + calendar_xml+ call_info + categories + recent_replies + action_result
