import sys

from content import ContentModule
from database.dbobjects import DBPage,DBPageBody
from BTrees import OOBTree
from BTrees.Length import Length

import ZODB

from wikiutil import compile_wiki_string,quoteWikiname,unquoteWikiname

CREATE_ACTION = 1
VIEW_ACTION = 2
DELETE_ACTION = 3
EDIT_ACTION = 4
DIFF_ACTION = 5
CREATE_DB_ACTION = 6
EDIT_DB_ACTION = 7
REVERSE_LINK_ACTION = 8

action_dict = {\
    "create":CREATE_ACTION,\
    "view":VIEW_ACTION,\
    "edit":EDIT_ACTION,\
    "diff":DIFF_ACTION,\
    "create_db":CREATE_DB_ACTION,\
    "edit_db":EDIT_DB_ACTION,\
    "show_reverse_link":REVERSE_LINK_ACTION,\
    }

def update_link_cache(pagename, env):
	if not env.db_facade.pages.has_key(pagename):
		return
	#print "running update_link_cache on page '%s'" % (pagename)
	if not env.db_facade.pages_link_cache.has_key(pagename):
		env.db_facade.pages_link_cache[pagename] = OOBTree.OOSet()

	dbpage = env.db_facade.pages[pagename]
	content = dbpage.GetRecentPageBody().data
	link_save_list = []
	compile_wiki_string(content,env.request.base_uri,"wiki/",env,link_save_list)
	for to_pagename in link_save_list:
		#print "link to '%s' on page '%s'" % (to_pagename,pagename)
		if not env.db_facade.pages_link_cache.has_key(to_pagename):
			env.db_facade.pages_link_cache[to_pagename] = OOBTree.OOSet()
		reverse_link_set = env.db_facade.pages_link_cache[to_pagename]
		if not pagename in reverse_link_set:
			reverse_link_set.insert(pagename)

def update_full_link_cache(env):
	for pagename in env.db_facade.pages.keys():
		update_link_cache(pagename,env)

def do_create(env, pagename, db_page):
    return "<action name=\"create\" pagename=\"%s\" pagelinkname=\"%s\"/>" % (pagename,quoteWikiname(pagename))

def do_view(env, pagename, db_page):
	data = db_page.GetRecentPageBody().data
	data = compile_wiki_string(data,env.request.base_uri,"wiki/",env)
	#update_link_cache(pagename,env)
	return "<action name=\"view\" pagename=\"%s\" pagelinkname=\"%s\"><page-data><![CDATA[%s]]></page-data></action>" % (pagename,quoteWikiname(pagename),data)
    
def do_edit(env, pagename, db_page):
	data = db_page.GetRecentPageBody().data
	return "<action name=\"edit\" pagename=\"%s\" pagelinkname=\"%s\"><page-data><![CDATA[%s]]></page-data></action>" % (pagename,quoteWikiname(pagename),data)
    
def do_diff(env, pagename, db_page):
    return "do_diff called on %s" % (pagename)

def do_edit_db(env,pagename,db_page):
	if not env.request.form.has_key('content'):
		return "<action name=\"error\" pagename=\"%s\" on=\"edit_db\"/>" % (pagename)
	content = env.request.form['content'].value
	dbpagebody = DBPageBody(None,db_page,content, env.db_facade.db)
	db_page.AddPageBody(dbpagebody,env.db_facade.db)
	#update_link_cache(pagename,env)
	update_full_link_cache(env)
	env.db_facade.Commit()
	get_transaction().commit()
	return "<action name=\"edit_db\" pagename=\"%s\" pagelinkname=\"%s\"/>" % (pagename,quoteWikiname(pagename))

def do_create_db(env,pagename,db_page):
	if not env.request.form.has_key('title'):
		return "<action name=\"error\" pagename=\"%s\" on=\"create_db\"/>" % (pagename)
	if not env.request.form.has_key('content'):
		return "<action name=\"error\" pagename=\"%s\" on=\"create_db\"/>" % (pagename)
	 # TODO Time info, Author Info
	title = env.request.form['title'].value
	content = env.request.form['content'].value
	dbpage = DBPage(None, title, env.db_facade.db)
	dbpagebody = DBPageBody(None,dbpage,content,env.db_facade.db)
	dbpage.AddPageBody(dbpagebody,env.db_facade.db)
	env.db_facade.pages[title] = dbpage
	pagename = title
	#update_link_cache(pagename,env)
	update_full_link_cache(env)
	env.db_facade.Commit()
	get_transaction().commit()
	return "<action name=\"create_db\" pagename=\"%s\" pagelinkname=\"%s\"/>" % (pagename,quoteWikiname(pagename))

reverse_links_xml_template = """
<action name="show_reverse_link" pagename="%s" pagelinkname="%s">
	<reverse_links>
		%s
	</reverse_links>
</action>
"""

reverse_link_xml_template = """<reverse_link pagename="%s" pagelinkname="%s"/>"""

def do_reverse_link(env,pagename,db_page):
	if not env.db_facade.pages_link_cache.has_key(pagename):
		update_full_link_cache(env)
		env.db_facade.Commit()
		get_transaction().commit()
	link_cache = env.db_facade.pages_link_cache[pagename]
	pagenames = []
	reverse_links = ""
	for link in link_cache:
		#print "Link on '%s' to '%s'" % (link,pagename)
		pagenames.append(link)
		reverse_links += reverse_link_xml_template % (link, quoteWikiname(link))
	return reverse_links_xml_template % (pagename,quoteWikiname(pagename),reverse_links)
	
embedded_dispatcher_dict = {\
    CREATE_ACTION: do_create,\
    VIEW_ACTION: do_view,\
    EDIT_ACTION: do_edit,\
    DIFF_ACTION: do_diff,\
    CREATE_DB_ACTION: do_create_db,\
    EDIT_DB_ACTION: do_edit_db,\
    REVERSE_LINK_ACTION : do_reverse_link,\
    }

class WikiModule(ContentModule):
	def GetTagName(self):
		return "wiki"
	def Run(self, application_id, env, module_uri):
		pagename = ""
		if module_uri == "" or module_uri == "/":
			pagename = "FrontPage"
		elif module_uri.startswith("/"):
			pagename = module_uri[1:]
		else:
			pagename = module_uri
		if pagename.startswith("/"):
			pagename = pagename[1:]
		pagename = unquoteWikiname(pagename)

		# TODO Is there any need to convert korean pagename?
        
		action = VIEW_ACTION
		dbpage = None

		# is there that page?
		if not env.db_facade.pages.has_key(pagename):
			#print "We cannot Get DBPAGE! : " + pagename
			# there are no page... so it's creation mode
			if env.request.form.has_key("action") and env.request.form["action"].value == "create_db":
				#print "So, Create it"
				action = CREATE_DB_ACTION
			else:
				#print "Please, Create it"
				action = CREATE_ACTION
		elif env.request.form.has_key("action"):
			#print "We got the page"
			# we have page!!!
			dbpage = env.db_facade.pages[pagename]
			#print "dbpage is the page: " + str(dbpage)
			# check action.
			if action_dict.has_key(env.request.form["action"].value):
				# it's embedded action
				action = action_dict[env.request.form["action"].value]
			else:
				# it's undefined action (external)
				# TODO Design call architecture
				# Now runs external action dispatcher
				pass
		else:
			#print "We got the page"
			# we have page!!!
			dbpage = env.db_facade.pages[pagename]
			#print "dbpage is the page: " + str(dbpage)
		# Now Add Call Info
		call_info = "<module-call-info><application-id>%s</application-id><module_uri>%s</module_uri></module-call-info>"\
			% (application_id, module_uri)
		# Now runs embedded action dispatcher
		if embedded_dispatcher_dict.has_key(action):
			return call_info + embedded_dispatcher_dict[action](env,pagename,dbpage)
		else:
			# TODO change this to Exception
			return "ERROR!!! "
		# TODO Change this to Exception
		return call_info + "Wiki Module Not Runned Correctly"

if __name__ ==  '__main__' :
	wiki_str = "123456789[789456123]78456123[abcdefghi]12345"
	print compile_wiki_string(wiki_str,"http://aaaaaa", "/wiki/")
 	
 
