import re
import string
import urllib
import config
# Thanks for MoinMoin
def quoteFilename(filename):
    safe = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ' + string.digits
    res = list(filename)
    for i in range(len(res)):
        c = res[i]
        if c not in safe:
            res[i] = '_%02x' % ord(c)
    return string.joinfields(res, '')

def unquoteFilename(filename):
    return urllib.unquote(string.replace(filename, '_', '%'))

quoteWikiname = quoteFilename
unquoteWikiname = unquoteFilename

escape_open_pattern = r"\[\["
escape_close_pattern = r"\]\]"

escape_open_special_zone_pattern = r"(\\\{)|(&\#92;\})"
escape_close_special_zone_pattern = r"(\\\})|(&\#92;\{)"

start_special_zone_pattern = r"{{{"
end_special_zone_pattern = r"}}}"

hide_pattern = r"\[\[\[[^\[|^\]]*\]\]\]"
link_pattern = r"\[[^\]]*\]"

total_pattern = "(%s)|(%s)|(%s)|(%s)|(%s)" % \
	(\
	start_special_zone_pattern,\
	end_special_zone_pattern,\
	escape_open_pattern,\
	escape_close_pattern,\
	link_pattern,\
	)

hide_tag_html_template = """
<script type="text/javascript">
	function __toggle_hide_zone%(unique_id)d()
	{
		elm = document.getElementById("hide_zone%(unique_id)d");
		if(elm.style.visibility == 'visible') __hide_hide_zone%(unique_id)d();
		else __show_hide_zone%(unique_id)d();
	}
	function __show_hide_zone%(unique_id)d()
	{
		elm = document.getElementById("hide_zone%(unique_id)d");
		elm.style.visibility='visible';
		elm.style.display="block";
	}
	function __hide_hide_zone%(unique_id)d()
	{
		elm = document.getElementById("hide_zone%(unique_id)d");
		elm.style.visibility='hidden';
		elm.style.display="none";
	}
</script>
<a href="javascript:__toggle_hide_zone%(unique_id)d();">
	%(button_name)s
</a>
<div class="wiki_content_hide" id="hide_zone%(unique_id)d">
%(hide_data)s
</div>
"""
quote_tag_html_template = """
<div class="wiki_content_quote">
%s
</div>
"""

id = 1

class SmartTagRenderer:
	def __init__(self, base_uri,module,blog_id,env,link_save_list):
		self.base_uri = base_uri
		self.module_id = module
		self.env = env
		self.blog_id = blog_id
		self.link_save_list = link_save_list
	
	def render_special_zone(self,hide_string):
		global id
		if hide_string.startswith("quote"):
			return quote_tag_html_template % (hide_string[5:])
		if hide_string.startswith("hide"):
			id += 1
			if hide_string[4] == '(':
				idx = hide_string.find(')')
				if idx != -1:
					return hide_tag_html_template % {"button_name":hide_string[5:idx],"unique_id":id,"hide_data":hide_string[idx+1:]}
			return hide_tag_html_template % {"button_name":"toggle hide","unique_id":id,"hide_data":hide_string[4:]}
		else:
			id += 1
			return hide_tag_html_template % {"button_name":"toggle hide","unique_id":id,"hide_data":hide_string}
	def render_escape_open(self):
		return "["
	def render_escape_close(self):
		return "]"
	def render_link(self,link_string):
		if link_string.startswith("http://"):
			#external link
			space_idx = link_string.find(" ")
			link_name = ""
			link_url = ""
			if space_idx == -1:
				link_name = link_string
				link_url = link_string
			else:
				link_name = link_string[space_idx+1:]
				link_url = link_string[:space_idx]
			return """<a href="%s" target="_new">%s</a>""" % (link_url,link_name)
		else:
			# internal link
			if link_string.startswith(":blog"):
				idx1 = link_string.find("(")
				idx2 = link_string.find(")")
				if idx1 == -1 or idx2 == -1:
					return "rendering: syntax error"
				else:
					try:
						blog_seq = int(link_string[idx1+1:idx2])
					except:
						return "rendering: blog seq is not integer"
					if self.env.db_facade.postings.has_key(blog_seq):
						posting = self.env.db_facade.postings[blog_seq]
						return """<a href="%s/%s/%d">%s</a>""" % (self.base_uri,self.blog_id,blog_seq,posting.title)
					else:
						return "rendering: post %d is not exist." % (blog_id)
			# find page.
			if self.env.db_facade.pages.has_key(link_string):
				if self.link_save_list != None:
					self.link_save_list.append(link_string)
				return """<a href="%s/%s/%s">%s</a>""" % (self.base_uri,self.module_id,quoteWikiname(link_string),link_string)
			else:
				return """%s<a href="%s/%s/%s">?</a>""" % (link_string,self.base_uri,self.module_id,quoteWikiname(link_string))
	
def compile_wiki_string(data, base_uri, module,env,link_save_list=None):
	tokens = re.split(total_pattern,data)
	compiled = ""
	special_zone_data = ""
	is_special_zone = False
	renderer = SmartTagRenderer(base_uri,module,config.wiki_render_blog_id,env,link_save_list)
	for token in tokens:
		#print "token: " + str(token)
		if token and re.search(start_special_zone_pattern,token):
			is_special_zone = True
		elif token and re.search(end_special_zone_pattern,token):
			compiled += renderer.render_special_zone(special_zone_data)
			special_zone_data = ""
			is_special_zone = False
		elif token and re.search(link_pattern,token):
			if is_special_zone == True:
				special_zone_data += renderer.render_link(token[1:-1])
			else:
				compiled += renderer.render_link(token[1:-1])
		elif token and re.search(escape_open_pattern,token):
			compiled += renderer.render_escape_open()
		elif token and re.search(escape_close_pattern,token):
			compiled += renderer.render_escape_close()
		elif token and is_special_zone == True:
			special_zone_data += token
		elif token:
			compiled += token
	compiled = compiled.replace("\\{","{")
	compiled = compiled.replace("\\}","}")
	compiled = compiled.replace("&#92;}","}")
	compiled = compiled.replace("&#92;{","{")
	
	return compiled

if __name__=='__main__':
	print "FIRST============"
	print compile_wiki_string("TEXT[1]\n[[[HIDE]]]TEXT2\n[[[[NOPROC]]]]dfasdf[[[hide\nsection]]]sda","base_uri","module")
	