class ApplicationDescriber:
	def __init__(self):
		self.APP_START = """<?xml version="1.0" encoding="utf-8"?><Application id="%s" action="%s">"""
		self.APP_END = """</Application>"""
		
		self.REQUEST_START = """<Request>"""
		self.REQUEST_ITEM = """<item key="%s" value="%s"/>"""
		self.REQUEST_END = """</Request>"""

		self.SESSION_START = """<Session>"""
		self.SESSION_ITEM = """<item key="%s" value="%s"/>"""
		self.SESSION_END = """</Session>"""

		self.result = ""
	
	def start_application(self,app_id,action):
		self.result += self.APP_START % (app_id, action)
	
	def end_application(self):
		self.result += self.APP_END
	
	def start_request(self):
		self.result += self.REQUEST_START
		
	def add_request_item(self, key, item):
		self.result += self.REQUEST_ITEM % (key, item)
	
	def end_request(self):
		self.result += self.REQUEST_END
		
	def start_session(self):
		self.result += self.SESSION_START
	
	def add_session_item(self, key, item):
		self.result += self.SESSION_ITEM % (key, item)
	
	def end_session(self):
		self.result += self.SESSION_END
	
	def add_result(self,result):
		self.result += result
		
	def get_result(self):
		return self.result
	

class ResultDescriber:
	def __init__(self):
		self.ERROR_TEMPLATE = """<SQLResult error="yes"> %s </SQLResult>"""
		self.RESULT_BEGIN = """<SQLResult name="%s"><TriedQuery><![CDATA[%s]]></TriedQuery>"""
		self.ROW_BEGIN = """<Row seq="%d">"""
		self.COL_BEGIN = """<Column seq="%d" name="%s"><![CDATA[%s]]>"""
		self.COL_END = """</Column>"""
		self.ROW_END = """</Row>"""
		self.NEST_RESULT = """%s"""
		self.RESULT_END = """</SQLResult>"""
		self.result = ""
		self.row_idx = 0
		self.col_idx = 0
		
	def start_result(self,name,query):
		self.result += self.RESULT_BEGIN % (name, query)
	
	def start_row(self):
		self.result += self.ROW_BEGIN % (self.row_idx)

	def start_column(self, key, data, escape):
		converted = ""
		
		if escape and (type(data) == type("") or type(data) == type(u"") ):
			converted = data.replace("<","&lt;").replace(">","&gt;")
		else:
			converted = data
		
		if type(converted) == type(""):
			converted = converted.decode("euc-kr").encode("utf-8")
		elif type(converted) == type(u""):
			converted = converted.encode("utf-8")
		
		self.result += self.COL_BEGIN % (self.col_idx, key, converted)
	
	def end_column(self):
		self.result += self.COL_END
		self.col_idx += 1

	def end_row(self):
		self.result += self.ROW_END
		self.row_idx += 1
		self.col_idx = 0
	
	def add_nest_result(self, result):
		self.result += self.NEST_RESULT % (result)	
	
	def end_result(self):
		self.result += self.RESULT_END
	
	def render_error(self,desc):
		return self.ERROR_TEMPLATE % (desc)
		
	def get_result(self):
		return self.result
		
	def render_empty(self,name):
		return """<SQLResult name="%s"/>""" %(name)
	